searchData={"items":[{"type":"behaviour","title":"Stdio","doc":"Stream standard I/O from system processes.","ref":"Stdio.html"},{"type":"behaviour","title":"Overview - Stdio","doc":"`Stdio` manages system processes. The process standard output (`stdout`)\nand standard error (`stderr`) are represented as `Stream`s of\n`t:stdio/0` tuples. `pipe!/4` reads standard input (`stdin`) from a\nbinary stream.\n\n* system processes run as [foreground\n  processes](https://jdebp.uk/FGA/unix-daemon-design-mistakes-to-avoid.html)\n\n* when the foreground process exits, daemonized and background\n  subprocesses are reliably terminated\n\n* system processes are optionally restricted using behaviours such as\n  `Stdio.Container`, `Stdio.Rootless` or `Stdio.Jail`","ref":"Stdio.html#module-overview"},{"type":"behaviour","title":"Streaming stdout/stderr - Stdio","doc":"A simple example of creating a stream using `Stdio.stream!/1`:\n\n    iex> Stdio.stream!(\"echo hello; echo world >&2\") |> Enum.to_list()\n    [stdout: \"hello\\n\", stderr: \"world\\n\", exit_status: 0]","ref":"Stdio.html#module-streaming-stdout-stderr"},{"type":"behaviour","title":"Piping to stdin - Stdio","doc":"An example of using a system process to transform a stream using\n`Stdio.pipe!/2`:\n\n    iex> [\"hello\\n\"] |> Stdio.pipe!(\"sed -u 's/hello/bye/'\") |> Enum.to_list()\n    [stdout: \"bye\\n\", exit_status: 0]\n\n> #### Note {: .info}\n> Process output may be buffered. The pipe example\n> unbuffers the output by using the `-u` option of\n> [sed(1)](https://man7.org/linux/man-pages/man1/sed.1.html).","ref":"Stdio.html#module-piping-to-stdin"},{"type":"behaviour","title":"Privileges - Stdio","doc":"> #### Warning {: .warning}\n> Some behaviours, notably `Stdio.Container` and `Stdio.Jail`,\n> require the system supervisor (not the beam process!) to be running as\n> the `root` user. See `setuid/0` for instructions for set up.\n\nBehaviours may change the root filesystem for the process. The default\n`chroot(2)` directory hierarchy can be created by running:\n\n    iex> Stdio.Container.make_chroot_tree!()\n    :ok","ref":"Stdio.html#module-privileges"},{"type":"behaviour","title":"Process Capabilities and Namespaces - Stdio","doc":"`Stdio` behaviours can restrict system process capabilities or isolate\nthe process using namespaces or jails.\n\nThe default implementation uses the standard `fork(2)`'ing process model.\n\nOther implementations are `Stdio.Container` (Linux\n[namespaces(7)](https://man7.org/linux/man-pages/man7/namespaces.7.html)),\n`Stdio.Rootless` (Linux\n[user_namespaces(7)](https://man7.org/linux/man-pages/man7/user_namespaces.7.html))\nand `Stdio.Jail` (FreeBSD [jail(2)](https://www.freebsd.org/cgi/man.cgi?jail(2))\n\n    iex> Stdio.stream!(\"id -u\", Stdio.Rootless) |> Enum.to_list()\n    [stdout: \"65534\\n\", exit_status: 0]","ref":"Stdio.html#module-process-capabilities-and-namespaces"},{"type":"behaviour","title":"Supervising System Processes - Stdio","doc":"`Stdio` processes can be supervised and restarted by a `Task.Supervisor`:\n\n    Supervisor.start_link(\n      [{Task.Supervisor, name: Init.TaskSupervisor}],\n      strategy: :one_for_one\n    )\n\n To create and supervise a system process:\n\n    require Logger\n\n    Task.Supervisor.start_child(\n      Init.TaskSupervisor,\n      fn ->\n        Stdio.stream!(\"nc -l 4545\")\n        |> Stream.each(fn\n          {:stdout, e} ->\n            Logger.info(e)\n\n          {:stderr, e} ->\n            Logger.error(e)\n\n          e ->\n            Logger.warn(\"#{inspect(e)}\")\n        end)\n        |> Stream.run()\n      end,\n      restart: :permanent\n    )\n\nIn another shell, connect to port 4545 and send some data:\n\n    $ nc localhost 4545\n    test\n    123\n    ^C\n\n    $ nc localhost 4545\n    netcat was restarted by the supervisor!\n    ^C\n\nIn `iex`:\n\n    05:40:26.277 [info]  test\n    05:40:27.669 [info]  123\n    05:40:28.044 [warning] {:exit_status, 0}\n    05:40:52.929 [info]  netcat was restarted by the supervisor!\n    05:40:53.451 [warning] {:exit_status, 0}","ref":"Stdio.html#module-supervising-system-processes"},{"type":"callback","title":"Stdio.init/1","doc":"Function to `fork(2)` a system subprocess from the supervisor created\nby the `c:task/1` callback.\n\nIn a PID namespace, this process will be the `init` process or PID 1.","ref":"Stdio.html#c:init/1"},{"type":"callback","title":"Stdio.onerror/1","doc":"Function run on operation error.\n\n`c:onerror/1` runs if any of the operations defined in `c:ops/1`\nreturns an error. An error is an error tuple:\n\n    {:error, :prx.posix()}\n\nThe return value of `c:onerror/1` is ignored. Exceptions within the\ncallback are not handled by default and may terminate the linked\nprocess.\n\nThe supervisor process for the task can be found by calling\n`:prx.parent/1`. For example, to signal the process group:\n\n    supervisor = :prx.parent(sh)\n    :prx.kill(supervisor, 0, :SIGTERM)","ref":"Stdio.html#c:onerror/1"},{"type":"callback","title":"Stdio.onexit/1","doc":"Function run during stream termination.\n\n`c:onexit/1` runs when the stream is closed. The function can perform\nclean up and signal any lingering processes. Returns `true` if\nsubprocesses were found to be running during cleanup.","ref":"Stdio.html#c:onexit/1"},{"type":"callback","title":"Stdio.ops/1","doc":"A sequence of `t:Stdio.Op.t()/0` instructions for a system process.\n\nOperations are defined in [prx](https://hexdocs.pm/prx/prx.html)\nbut any module can be called by specifying the module name.","ref":"Stdio.html#c:ops/1"},{"type":"function","title":"Stdio.pipe!/4","doc":"Pipe data into the standard input of a system process.\n\nInput is a stream of binaries. Empty binaries are ignored.\n\nSee `stream!/3` for arguments.","ref":"Stdio.html#pipe!/4"},{"type":"function","title":"Examples - Stdio.pipe!/4","doc":"iex> Stream.unfold(1000, fn 0 -> nil; n -> {\"#{n}\\n\", n - 1} end)\n    ...> |> Stdio.pipe!(\"grep --line-buffered 442\")\n    ...> |> Enum.to_list()\n    [stdout: \"442\\n\", exit_status: 0]\n\n    iex> Stdio.stream!(\"echo test; kill $$\")\n    iex> |> Stream.transform(<<>>, &Stdio.Stream.stdout_to_stderr/2)\n    iex> |> Stdio.pipe!(\"sed -u 's/t/_/g'\")\n    iex> |> Enum.to_list()\n    [stdout: \"_es_\\n\", exit_status: 0]\n\n    iex> [<<>>, <<\"a\">>, <<>>] |> Stdio.pipe!(\"cat\") |> Enum.to_list()\n    [stdout: \"a\", exit_status: 0]","ref":"Stdio.html#pipe!/4-examples"},{"type":"function","title":"Stdio.reap/2","doc":"Terminate descendents of a supervisor process.","ref":"Stdio.html#reap/2"},{"type":"function","title":"Stdio.reap/3","doc":"Terminate descendents of a process.\n\nreap signals subprocesses of a process identified by PID.\n\nBackground and daemonized subprocesses will also be terminated if\nsupported by the platform.","ref":"Stdio.html#reap/3"},{"type":"function","title":"Stdio.setuid/0","doc":"Request system supervisor runs as root.\n\nSome system calls require the process to be UID 0. These calls typically\nare operations like mounting filesystems or creating new namespaces\n(excluding user namespaces such as `Stdio.Rootless`) or putting a\nprocess into a `jail(2)` or a `chroot(2)`.\n\n>","ref":"Stdio.html#setuid/0"},{"type":"function","title":"Warning {: .warning} - Stdio.setuid/0","doc":"> Enabling privileges is a global setting: all system supervisor\n> processes will run as UID 0.","ref":"Stdio.html#setuid/0-warning-warning"},{"type":"function","title":"Setuid Binary - Stdio.setuid/0","doc":"Probably the simplest method for production use is adding the setuid\nbit to the `prx` binary (see `:prx_drv.progname/0`):\n\n* create a group with privileges to run the supervisor (optional)\n\n    \t# example: create an elixir group\n    \tgroupadd elixir\n\n* find the location of the binary\n\n    \t:prx_drv.progname()\n\n* set the setuid bit\n\n    \tsudo chown root:elixir /path/to/priv/prx\n    \tsudo chmod 4750 /path/to/priv/prx","ref":"Stdio.html#setuid/0-setuid-binary"},{"type":"function","title":"Using a Setuid Exec Chain - Stdio.setuid/0","doc":"`prx` can be configured to use a setuid helper program such as `sudo(8)`\nor `doas(1)`.\n\n`Stdio.setuid/0` is a noop if the `prx` binary is setuid.\n\nBefore launching any supervisors, run:\n\n     Stdio.setuid()","ref":"Stdio.html#setuid/0-using-a-setuid-exec-chain"},{"type":"function","title":"sudo(8) - Stdio.setuid/0","doc":"$ sudo visudo\n\t\telixiruser ALL = NOPASSWD: /path/to/prx/priv/prx\n\t\tDefaults!/path/to/prx/priv/prx !requiretty","ref":"Stdio.html#setuid/0-sudo-8"},{"type":"function","title":"doas(8) - Stdio.setuid/0","doc":"permit nopass setenv { ENV PS1 SSH_AUTH_SOCK } :elixirgroup","ref":"Stdio.html#setuid/0-doas-8"},{"type":"function","title":"Errors - Stdio.setuid/0","doc":"#### `shell process exited with reason: {:error, {:error, :eagain}}`\n\nCheck `prx` is executable:\n\n```\n$ ./deps/prx/priv/prx -h\nprx 0.40.5\nusage: prx -c   [ ]\n```\n\n#### `error attempting to fork subprocess: insufficient permissions (eperm)`\n\nVerify `prx` can be run using `sudo`:\n\n```\n$ sudo ./deps/prx/priv/prx -h\nprx 0.40.5\nusage: prx -c   [ ]\n```","ref":"Stdio.html#setuid/0-errors"},{"type":"function","title":"Stdio.stream!/3","doc":"Stream standard I/O from system processes.","ref":"Stdio.html#stream!/3"},{"type":"function","title":"command - Stdio.stream!/3","doc":"Commands are strings interpreted by `/bin/sh` or list of strings\n(an argv) passed directly to `execve(2)`.\n\n> #### Warning {: .warning}\n> When using an argv, $PATH is not consulted. The path to the\n> executable must be provided as the first argument.","ref":"Stdio.html#stream!/3-command"},{"type":"function","title":"behaviour - Stdio.stream!/3","doc":"The process behaviour can be customized by passing in:\n\n* a module implementing the callbacks\n\n*  a keyword list with functions of type `t:phase/0`\n\n* a tuple consisting of a module and a list of `t:phase/0` functions\n  to override the module callbacks","ref":"Stdio.html#stream!/3-behaviour"},{"type":"function","title":"config - Stdio.stream!/3","doc":"For configuration options, see `t:config/0`.","ref":"Stdio.html#stream!/3-config"},{"type":"function","title":"Examples - Stdio.stream!/3","doc":"iex> Stdio.stream!(\"echo test\") |> Enum.to_list()\n    [stdout: \"test\\n\", exit_status: 0]\n\n    iex> Stdio.stream!([\"/bin/echo\", \"test\"]) |> Enum.to_list()\n    [stdout: \"test\\n\", exit_status: 0]\n\n    iex> Stdio.stream!(\"ip --oneline -4 addr show dev lo\", Stdio.Container) |> Enum.to_list()\n    [exit_status: 0]\n\n    iex> Stdio.stream!(\"ip --oneline -4 addr show dev lo\", Stdio.Container, net: :host) |> Enum.to_list()\n    [\n        stdout: \"1: lo    inet 127.0.0.1/8 scope host lo\\\\       valid_lft forever preferred_lft forever\\n\",\n        exit_status: 0\n    ]\n\n    iex> Stdio.stream!(\"pwd\", {Stdio.Process, ops: fn _ -> [{:chdir, [\"/tmp\"]}] end}) |> Enum.to_list()\n    [stdout: \"/tmp\\n\", exit_status: 0]","ref":"Stdio.html#stream!/3-examples"},{"type":"function","title":"Stdio.supervise/3","doc":"Configure a task to be a system process supervisor.\n\nThe task:\n\n* becomes an init process (dependent on platform)\n\n* sets the process name to `supervise`\n\n* enables flow control for child stdout/stderr\n\n* sets the shutdown/reaper behaviour\n\n* terminates child processes if beam exits\n\n* restricts the task to a limited set of calls required for supervision","ref":"Stdio.html#supervise/3"},{"type":"function","title":"Stdio.supervisor/2","doc":"Create a supervisor task\n\nReturns a `t:Stdio.t/0` configured to act as a system process supervisor.","ref":"Stdio.html#supervisor/2"},{"type":"callback","title":"Stdio.task/1","doc":"Function to create a system supervisor for the `Stdio` stream.\n\nCreate a task using `:prx.fork/0` and configured as a process\nsupervisor.\n\nSee `supervise/1` for the default operations.","ref":"Stdio.html#c:task/1"},{"type":"function","title":"Stdio.with_supervisor/2","doc":"Use a previously created supervisor process.\n\nThe supervisor is not shutdown when the stream exits.","ref":"Stdio.html#with_supervisor/2"},{"type":"function","title":"Examples - Stdio.with_supervisor/2","doc":"iex> {:ok, supervisor} = Stdio.supervisor(:noshutdown)\n    iex> Stdio.stream!(\"echo test\", Stdio.with_supervisor(Stdio.Process, supervisor)) |> Enum.to_list()\n    [stdout: \"test\\n\", exit_status: 0]\n    iex> Stdio.stream!(\"echo test\", Stdio.with_supervisor(Stdio.Process, supervisor)) |> Enum.to_list()\n    [stdout: \"test\\n\", exit_status: 0]\n    iex> :prx.stop(supervisor.init)\n\n```mermaid\ngraph LR\n    B([beam]) --> S[Supervise]\n    S -.-> I0[[sh]]\n    S --> I1[[sh]]\n    I0 -.-> C0[[echo test]]\n    I1 --> C1[[echo test]]\n```","ref":"Stdio.html#with_supervisor/2-examples"},{"type":"type","title":"Stdio.config/0","doc":"Configuration options for callbacks:","ref":"Stdio.html#t:config/0"},{"type":"type","title":"net - Stdio.config/0","doc":"Defines the network capability for behaviours with support for network\nnamespaces.\n\n* none: process does not have access to network\n\n* host: process shares the host's network namespace","ref":"Stdio.html#t:config/0-net"},{"type":"type","title":"environ - Stdio.config/0","doc":"Set the process environment using a list of `=` separated strings.\n\n    iex> Stdio.stream!([\"/usr/bin/env\"], Stdio.Process, environ: [\"TERM=dumb\", \"PATH=/bin\"]) |> Enum.to_list()\n    [stdout: \"TERM=dumb\\nPATH=/bin\\n\", exit_status: 0]","ref":"Stdio.html#t:config/0-environ"},{"type":"type","title":"uid - Stdio.config/0","doc":"> **Requires `setuid/0`**\n\nSet the UID of the subprocess. If not set, a UID is generated by\nthe behaviour.","ref":"Stdio.html#t:config/0-uid"},{"type":"type","title":"gid - Stdio.config/0","doc":"> **Requires `setuid/0`**\n\nSet the GID of the subprocess. If not set, a GID is generated by\nthe behaviour.","ref":"Stdio.html#t:config/0-gid"},{"type":"type","title":"groups - Stdio.config/0","doc":"> **Requires `setuid/0`**\n\nSet a list of additional groups (a list of GIDs) for the subprocess. By\ndefault, additional groups are dropped.","ref":"Stdio.html#t:config/0-groups"},{"type":"type","title":"fstab - Stdio.config/0","doc":"> **Requires `setuid/0`**\n\nA list of directories to bind mount inside of the container\nnamespace. Relative paths are resolved relative to the application\n`priv` directory.","ref":"Stdio.html#t:config/0-fstab"},{"type":"type","title":"path - Stdio.config/0","doc":"> **May require `setuid/0`**\n\nSets the working directory of the subprocess. Behaviours changing the\nroot directory use `path` as the root.","ref":"Stdio.html#t:config/0-path"},{"type":"type","title":"hostname - Stdio.config/0","doc":"> **Requires `setuid/0`**\n\nSets the hostname of the process.","ref":"Stdio.html#t:config/0-hostname"},{"type":"type","title":"setuid - Stdio.config/0","doc":"If set to `false` (the default), drops privileges before running\nthe command.","ref":"Stdio.html#t:config/0-setuid"},{"type":"type","title":"priority - Stdio.config/0","doc":"Sets the process priority (default: 0).","ref":"Stdio.html#t:config/0-priority"},{"type":"type","title":"rlimit - Stdio.config/0","doc":"A map containing `setrlimit(2)` settings. The default disables core dumps.","ref":"Stdio.html#t:config/0-rlimit"},{"type":"type","title":"Stdio.phase/0","doc":"Phases are functions defining how a process is forked, configured\nand supervised.\n\nPhases can be implemented as a behaviour or as a list of functions.\n\n```mermaid\nstateDiagram-v2\n    direction LR\n    [*] --> Supervisor: fork(2)\n    Supervisor --> Subprocess: fork(2)\n    Subprocess --> Terminate: stream\n    Terminate --> [*]\n    state Supervisor {\n        direction LR\n        [*] --> task\n        task --> [*]\n    }\n    state Subprocess {\n        state result < >\n        direction LR\n        [*] --> init\n        init --> ops\n        ops --> result\n        result --> [*]: ok\n        result --> onerror: {error, posix()}\n    }\n    state Terminate {\n        direction LR\n        onexit --> [*]\n    }\n```","ref":"Stdio.html#t:phase/0"},{"type":"type","title":"task - Stdio.phase/0","doc":"Returns a `t::prx.task/0` system process configured to act as a system\nprocess supervisor.","ref":"Stdio.html#t:phase/0-task"},{"type":"type","title":"init - Stdio.phase/0","doc":"Forks and initializes a subprocess of the system process\nsupervisor. When using namespaces, this process will be PID 1 or the\ncontainer `init` process.","ref":"Stdio.html#t:phase/0-init"},{"type":"type","title":"ops - Stdio.phase/0","doc":"Provides a sequence of system calls to run on the subprocess.","ref":"Stdio.html#t:phase/0-ops"},{"type":"type","title":"onerror - Stdio.phase/0","doc":"Function called to clean up if any of the operations defined in\n`c:ops/1` fail.","ref":"Stdio.html#t:phase/0-onerror"},{"type":"type","title":"onexit - Stdio.phase/0","doc":"Function called when the stream is closed.","ref":"Stdio.html#t:phase/0-onexit"},{"type":"type","title":"Stdio.pipeline/0","doc":"","ref":"Stdio.html#t:pipeline/0"},{"type":"type","title":"Stdio.stdio/0","doc":"Tuples containing the process stdout, stderr and termination status.\n\n* `:stdout`: standard output\n\n* `:stderr`: standard error\n\n* `:exit_status`: process has exited with status value\n\n* `:termsig`: the termination signal if the process exited due to a signal","ref":"Stdio.html#t:stdio/0"},{"type":"type","title":"Stdio.t/0","doc":"","ref":"Stdio.html#t:t/0"},{"type":"module","title":"Stdio.Container","doc":"Namespaced Linux processes\n\nRun a process in a Linux\n[namespace](https://man7.org/linux/man-pages/man7/namespaces.7.html).","ref":"Stdio.Container.html"},{"type":"module","title":"Privileges - Stdio.Container","doc":"To use this behaviour, the system process supervisor must have root\nprivileges. These privileges are dropped before running the command.\n\nRoot privileges can be enabled by setting the setuid bit on the `prx`\nexecutable or by running it under a program like `sudo(8)`.","ref":"Stdio.Container.html#module-privileges"},{"type":"module","title":"Operations - Stdio.Container","doc":"> #### Note {: .info}\n> To mount a namespaced root (chroot) directory, the chroot directory\n> structure must be created before using this behaviour.\n>\n> See `make_chroot_tree!/0` and `make_chroot_tree!/1`.\n\nSee `t:Stdio.config/0` for configuration options.\n\n* creates a process in a new UTS, IPC, PID, mount and by default,\n  net namespace\n\n* creates a new session\n\n* sets the process priority [:priority=0]\n\n* bind mounts as read-only /bin, /sbin, /usr, /lib and the list\n  specified in `fstab` into the container mount namespace\n\n* mounts /tmp and /home as tmpfs filesystems\n\n* changes the mount namespace root directory to the chroot\n\n* sets resource limits defined in the `rlimit` option [:rlimit=disable\n  coredumps]\n\n* sends the process a SIGKILL if the parent process exits\n\n* sets additional groups as specified in the `group` option\n  [:groups=remove additional groups]\n\n* drops privileges to the value of `uid` and `gid` or a high UID system\n  user [:uid/:gid=65536-131071]\n\n* disables the ability of the process to escalate privileges [:setuid=false]\n\n> #### Warning {: .warning}\n> The generated UID/GID may overlap with existing users.","ref":"Stdio.Container.html#module-operations"},{"type":"module","title":"Examples - Stdio.Container","doc":"iex> Stdio.stream!(\"pstree\", Stdio.Container) |> Enum.to_list()\n    [stdout: \"sh---pstree\\n\", exit_status: 0]","ref":"Stdio.Container.html#module-examples"},{"type":"function","title":"Stdio.Container.make_chroot_tree!/1","doc":"Create the container root directory structure.\n\nCreates the directory structure set in the application environment. The\ndefault is:\n\n    # the default is set to the application priv directory: priv/root\n    config :stdio,\n      path: \"/tmp/root\"","ref":"Stdio.Container.html#make_chroot_tree!/1"},{"type":"module","title":"Stdio.Jail","doc":"Jailed FreeBSD processes\n\nRuns a process in a\n[jail(2)](https://www.freebsd.org/cgi/man.cgi?jail(2)).","ref":"Stdio.Jail.html"},{"type":"module","title":"Privileges - Stdio.Jail","doc":"To use this behaviour, the system process supervisor must have root\nprivileges. These privileges are dropped before running the command.\n\nSee `Stdio.setuid/0`.","ref":"Stdio.Jail.html#module-privileges"},{"type":"module","title":"sysctl(8) - Stdio.Jail","doc":"[sysctl(8)](https://www.freebsd.org/cgi/man.cgi?sysctl(8)) settings\ncontrol the behaviour of the jail. For example, to allow ping/traceroute\nfrom the jail:\n\n    sysctl security.jail.allow_raw_sockets=1\n\nSee [jail(8)](https://www.freebsd.org/cgi/man.cgi?jail(8)).","ref":"Stdio.Jail.html#module-sysctl-8"},{"type":"module","title":"Operations - Stdio.Jail","doc":"See `t:Stdio.config/0` for configuration options.\n\n* creates a new session\n\n* sets the process priority [:priority=0]\n\n* puts the process into a `jail(2)`\n\n* sets resource limits [:rlimit=coredumps disabled]\n\n* sets additional groups [:groups=additional groups removed]\n\n* drops privileges to the value of `uid` and `gid` or a high UID system\n  user [:uid/:gid=65536-131071]\n\n* disables the capability to elevate privileges [:setuid=false]\n\n> #### Warning {: .warning}\n> The generated UID/GID may overlap with existing users.","ref":"Stdio.Jail.html#module-operations"},{"type":"module","title":"Examples - Stdio.Jail","doc":"iex> Stdio.stream!([\"./echo\", \"test\"], Stdio.Jail, path: \"/rescue\")\n    ...> |> Enum.to_list()\n    [stdout: \"test\\n\", exit_status: 0]\n\n    iex> Stdio.stream!(\n    ...> [\"sh\", \"-c\", \"export PATH=/; ping -c 1 127.0.0.1 | head -1\"],\n    ...> Stdio.Jail,\n    ...> uid: 0, path: \"/rescue\", setuid: true, net: :host\n    ...>) |> Enum.to_list()\n    [stdout: \"PING 127.0.0.1 (127.0.0.1): 56 data bytes\\n\", exit_status: 0]","ref":"Stdio.Jail.html#module-examples"},{"type":"module","title":"Stdio.Op","doc":"Run a sequence of system calls on a process","ref":"Stdio.Op.html"},{"type":"function","title":"Stdio.Op.task!/4","doc":"Creates and configures a new process from the supervisor process.","ref":"Stdio.Op.html#task!/4"},{"type":"type","title":"Stdio.Op.option/0","doc":"Options to modify the behavior of an operation:\n\n* `state`: pass `ok` result as the first parameter to the next\n   operation (default: false)\n\n* `errexit`: abort operations on error (default: true)\n\n* `transform`: abort operations on error (default: true)","ref":"Stdio.Op.html#t:option/0"},{"type":"type","title":"Stdio.Op.t/0","doc":"Operations consist of a list of tuples containing:\n\n* an optional module name (default: `:prx`)\n* a function name\n* a list of arguments\n* an optional list of options to modify the behaviour of the operation\n  (see `t:option`)","ref":"Stdio.Op.html#t:t/0"},{"type":"type","title":"Examples - Stdio.Op.t/0","doc":"{:setresuid, [1000,1000,1000]}\n    {:prx, :setresuid, [1000,1000,1000]}\n    {:prx, :setresuid, [1000,1000,1000], errexit: false}","ref":"Stdio.Op.html#t:t/0-examples"},{"type":"module","title":"Stdio.Process","doc":"Supervised system processes\n\n`fork(2)` new system processes.","ref":"Stdio.Process.html"},{"type":"module","title":"Privileges - Stdio.Process","doc":"No additional privileges required. If the `Stdio` supervisor process\nis running as root, supervised processes will by default drop to an\nunprivileged user.","ref":"Stdio.Process.html#module-privileges"},{"type":"module","title":"Operations - Stdio.Process","doc":"See `t:Stdio.config/0` for configuration options.\n\n* creates a new session\n\n* sets the process priority [:priority=0]\n\n* sets resource limits defined in the `rlimit` option [:rlimit=coredumps\n  disabled]\n\n* sends the process a SIGKILL if the parent process exits\n\nIf the system process is running with root privileges:\n\n* sets additional groups as specified in the `group` option\n  [:groups=additional groups removed]\n\n* drops privileges to the value of `uid` and `gid` or a high UID system\n  [:uid/gid=65536-131071]\n\n* disables the ability of the process to escalate privileges [:setuid=false]\n\n> #### Warning {: .warning}\n> The generated UID/GID may overlap with existing users.","ref":"Stdio.Process.html#module-operations"},{"type":"module","title":"Examples - Stdio.Process","doc":"iex> Stdio.stream!(\n    ...> \"ping -q -c 1 127.0.0.1 | grep -o PING\",\n    ...> Stdio.Process,\n    ...> setuid: true\n    ...> ) |> Enum.to_list()\n    [stdout: \"PING\\n\", exit_status: 0]","ref":"Stdio.Process.html#module-examples"},{"type":"function","title":"Stdio.Process.alive?/1","doc":"Test if a Stdio process is running","ref":"Stdio.Process.html#alive?/1"},{"type":"function","title":"Stdio.Process.signal/3","doc":"Signal a process by process group, falling back to PID","ref":"Stdio.Process.html#signal/3"},{"type":"module","title":"Stdio.ProcessTree","doc":"System process supervisor tree.","ref":"Stdio.ProcessTree.html"},{"type":"function","title":"Stdio.ProcessTree.task/1","doc":"","ref":"Stdio.ProcessTree.html#task/1"},{"type":"type","title":"Stdio.ProcessTree.t/0","doc":"","ref":"Stdio.ProcessTree.html#t:t/0"},{"type":"module","title":"Stdio.Procfs","doc":"Read data from [procfs](https://man7.org/linux/man-pages/man5/proc.5.html).","ref":"Stdio.Procfs.html"},{"type":"function","title":"Stdio.Procfs.children/2","doc":"Get descendents of a process.\n\nChild processes are enumerated by reading `proc(5)`:\n\n* if the Linux kernel was compiled with `CONFIG_PROC_CHILDREN`, the\n  `/proc/[pid]/task/[pid]/children` file is read\n\n* otherwise, `children/1` falls back to walking /proc","ref":"Stdio.Procfs.html#children/2"},{"type":"function","title":"Examples - Stdio.Procfs.children/2","doc":"iex> Stdio.Procfs.children(0) |> Enum.sort() |> List.first()\n    1","ref":"Stdio.Procfs.html#children/2-examples"},{"type":"function","title":"Stdio.Procfs.ps/1","doc":"Return a snapshot of the current system processes.\n\nGenerates a snapshot of running system processes by walking the procfs\nfilesystem (usually mounted as `/proc`).\n\nThe returned value is a map list of key/value pairs read from\n`/proc/[pid]/status`.","ref":"Stdio.Procfs.html#ps/1"},{"type":"function","title":"Examples - Stdio.Procfs.ps/1","doc":"iex> pid = Stdio.Procfs.ps() |> Enum.sort(&(&1[\"ppid\"] < &2[\"ppid\"])) |> List.first()\n    iex> pid[\"pid\"]\n    \"1\"","ref":"Stdio.Procfs.html#ps/1-examples"},{"type":"type","title":"Stdio.Procfs.t/0","doc":"","ref":"Stdio.Procfs.html#t:t/0"},{"type":"module","title":"Stdio.Rootless","doc":"Linux processes running in a user namespace.\n\nProof of concept behaviour to run \"rootless\" Linux\nprocesses or processes that have root privileges in a [user\nnamespace](https://man7.org/linux/man-pages/man7/user_namespaces.7.html).","ref":"Stdio.Rootless.html"},{"type":"module","title":"Privileges - Stdio.Rootless","doc":"No extra privileges are required but user namespaces must be enabled\non your platform:\n\n    $ sysctl kernel.unprivileged_userns_clone\n    kernel.unprivileged_userns_clone = 1\n\n    # to enable:\n    $ sysctl -w kernel.unprivileged_userns_clone=1","ref":"Stdio.Rootless.html#module-privileges"},{"type":"module","title":"Operations - Stdio.Rootless","doc":"> #### Note {: .info}\n> To mount a namespaced root (chroot) directory, the chroot directory\n> structure must be created before using this behaviour.\n>\n> See `Stdio.Container.make_chroot_tree!/0` and\n> `Stdio.Container.make_chroot_tree!/1`.\n\nSee `t:Stdio.config/0` for configuration options.","ref":"Stdio.Rootless.html#module-operations"},{"type":"module","title":"Examples - Stdio.Rootless","doc":"iex> Stdio.stream!(\"pstree\", Stdio.Rootless) |> Enum.to_list()\n    [stdout: \"Supervise---sh---pstree\\n\", exit_status: 0]\n\n    iex(1)> Stdio.stream!(\"ip --oneline -4 addr show dev lo\", Stdio.Rootless, net: :host, setuid: true) |> Enum.to_list()\n    [\n      stdout: \"1: lo    inet 127.0.0.1/8 scope host lo\\\\       valid_lft forever preferred_lft forever\\n\",\n      exit_status: 0\n    ]","ref":"Stdio.Rootless.html#module-examples"},{"type":"module","title":"Stdio.Stream","doc":"Stream standard I/O from system processes","ref":"Stdio.Stream.html"},{"type":"function","title":"Stdio.Stream.ratelimit/3","doc":"Rate limits a stream by discarding elements exceeding a threshold for\nthe remainder of the window.\n\n`ratelimit/3` works with any `Enumerable.t`. For working with streams\ngenerated by `Stdio`, see `ratelimit/4`.","ref":"Stdio.Stream.html#ratelimit/3"},{"type":"function","title":"Examples - Stdio.Stream.ratelimit/3","doc":"iex> Stream.unfold(1_000, fn 0 -> nil; n -> {n, n-1} end)\n    ...> |> Stdio.Stream.ratelimit(2, 10_000)\n    ...> |> Enum.to_list()\n    [1000, 999]","ref":"Stdio.Stream.html#ratelimit/3-examples"},{"type":"function","title":"Stdio.Stream.ratelimit/4","doc":"Rate limits a stream by calling a function on each event.\n\nThe function is a reducer for `Stream.transform/3` which is passed\nthe rate limit state in the accumulator:\n\n* t: window start time\n\n* limit: window threshold\n\n* ms: window duration in milliseconds\n\nThe function maintains a count of matching events and enables rate\nlimiting when the threshold is exceeded for the window.\n\nSee `stdio_block/2` and `stdio_drop/2` for functions which can apply\nback pressure to or drop events from `Stdio.stream!/1` or `Stdio.pipe!/2`.","ref":"Stdio.Stream.html#ratelimit/4"},{"type":"function","title":"Stdio.Stream.stdio_block/2","doc":"Apply back pressure on `Stdio.stream!/1` or `Stdio.pipe!/2` using\n`ratelimit/4` by blocking further reads if the rate of stdout/stderr\nexceeds the threshold.\n\nOutput accumulates until the process pipe buffer is full. Further\nwrites are blocked.\n\nControl events (process exit and termination signals) are not counted\nagainst the threshold.\n\n> #### Warning {: .warning}\n> When the rate limit is reached, the stream is blocked for the\n> remainder of the window, even if the system process has exited.\n\nTo see how it works, try running:\n\n    require Logger\n    Stdio.stream!(\"while :; do date; done\")\n    |> Stdio.Stream.ratelimit(1, 5_000, &Stdio.Stream.stdio_block/2)\n    |> Stream.each(fn t -> t |> inspect() |> Logger.info() end)\n    |> Enum.take(15)\n\n\nThe output from Logger will be spaced 5 seconds apart. The data from\n`Stdio.stream!/1` is buffered:\n\n    00:11:56.410 [info]  {:stdout, \"Thu Jul  7 00:11:51 EDT 2022\\n\"}\n    00:12:01.410 [info]  {:stdout, \"Thu Jul  7 00:11:51 EDT 2022\\n\"}\n    00:12:06.412 [info]  {:stdout, \"Thu Jul  7 00:11:51 EDT...\"}","ref":"Stdio.Stream.html#stdio_block/2"},{"type":"function","title":"Examples - Stdio.Stream.stdio_block/2","doc":"iex> Stdio.stream!(\"echo 1\")\n    ...> |> Stdio.Stream.ratelimit(1, 2_000, &Stdio.Stream.stdio_block/2)\n    ...> |> Enum.to_list()\n    [stdout: \"1\\n\", exit_status: 0]","ref":"Stdio.Stream.html#stdio_block/2-examples"},{"type":"function","title":"Stdio.Stream.stdio_drop/2","doc":"Rate limit output of `Stdio.stream!/1` or `Stdio.pipe!/2` using\n`ratelimit/4` by dropping events if the rate of stdout/stderr exceeds\nthe threshold.\n\nControl events (process exit and termination signals) are always\npassed through.\n\nTo see how it works, try running:\n\n    require Logger\n    Stdio.stream!(\"while :; do date; done\")\n    |> Stdio.Stream.ratelimit(1, 5_000, &Stdio.Stream.stdio_drop/2)\n    |> Stream.each(fn t -> t |> inspect() |> Logger.info() end)\n    |> Enum.take(15)\n\nThe output from Logger will be spaced 5 seconds apart. Rate limited\ndata from `Stdio.stream!/1` is dropped:\n\n    00:17:22.058 [info]  {:stdout, \"Thu Jul  7 00:17:22 EDT 2022\\n\"}\n    00:17:27.059 [info]  {:stdout, \"Thu Jul  7 00:17:27 EDT 2022\\n\"}\n    00:17:32.060 [info]  {:stdout, \"Thu Jul  7 00:17:32 EDT 2022\\n\"}\n    00:17:37.061 [info]  {:stdout, \"Thu Jul  7 00:17:37 EDT 2022\\n\"}","ref":"Stdio.Stream.html#stdio_drop/2"},{"type":"function","title":"Examples - Stdio.Stream.stdio_drop/2","doc":"iex> Stdio.stream!(\"echo 1\")\n    ...> |> Stdio.Stream.ratelimit(1, 2_000, &Stdio.Stream.stdio_drop/2)\n    ...> |> Enum.to_list()\n    [stdout: \"1\\n\", exit_status: 0]","ref":"Stdio.Stream.html#stdio_drop/2-examples"},{"type":"function","title":"Stdio.Stream.stdout_to_stderr/2","doc":"Combine standard error into the standard output of `Stdio.stream!/1`\nor `Stdio.pipe!/2` and emit a list of binaries.\n\nExit status and termination signals are not included in the output.","ref":"Stdio.Stream.html#stdout_to_stderr/2"},{"type":"function","title":"Examples - Stdio.Stream.stdout_to_stderr/2","doc":"iex> Stdio.stream!(\"echo output; echo error 1>&2; exit 1\")\n    ...> |> Stream.transform([], &Stdio.Stream.stdout_to_stderr/2)\n    ...> |> Enum.to_list()\n    [\"output\\n\", \"error\\n\"]","ref":"Stdio.Stream.html#stdout_to_stderr/2-examples"},{"type":"type","title":"Stdio.Stream.t/0","doc":"","ref":"Stdio.Stream.html#t:t/0"},{"type":"behaviour","title":"Stdio.Syscall","doc":"System call portability for operating systems.","ref":"Stdio.Syscall.html"},{"type":"function","title":"Stdio.Syscall.disable_setuid/0","doc":"The default implementation for `c:disable_setuid/0`\n\nDisabling the ability for the process to escalate privileges is not\navailable on this platform.","ref":"Stdio.Syscall.html#disable_setuid/0"},{"type":"callback","title":"Stdio.Syscall.disable_setuid/0","doc":"Operations required to disable setuid.","ref":"Stdio.Syscall.html#c:disable_setuid/0"},{"type":"function","title":"Stdio.Syscall.os/0","doc":"Get the syscall implementation for this platform.\n\nThe default implementation is based on the operating system.\n\nSet the implementation by configuring the application environment:\n\n    import Config\n\n    config :stdio, :syscall, Stdio.Syscall.Linux","ref":"Stdio.Syscall.html#os/0"},{"type":"function","title":"Stdio.Syscall.procfs/0","doc":"The default implementation for `c:procfs/0`","ref":"Stdio.Syscall.html#procfs/0"},{"type":"callback","title":"Stdio.Syscall.procfs/0","doc":"Path to procfs file system","ref":"Stdio.Syscall.html#c:procfs/0"},{"type":"function","title":"Stdio.Syscall.reap/2","doc":"Terminate descendents of a supervisor process.","ref":"Stdio.Syscall.html#reap/2"},{"type":"callback","title":"Stdio.Syscall.reap/2","doc":"Terminate descendents of a supervisor process.","ref":"Stdio.Syscall.html#c:reap/2"},{"type":"function","title":"Stdio.Syscall.reap/3","doc":"Terminate descendents of a process.\n\nSignals the process group for each child process.","ref":"Stdio.Syscall.html#reap/3"},{"type":"callback","title":"Stdio.Syscall.reap/3","doc":"Terminate a process subtree (descendents of a child process) of the supervisor.","ref":"Stdio.Syscall.html#c:reap/3"},{"type":"function","title":"Stdio.Syscall.set_pdeathsig/0","doc":"The default implementation for `c:set_pdeathsig/0`\n\nSignalling a process when the parent has exited is not available on\nthis platform.","ref":"Stdio.Syscall.html#set_pdeathsig/0"},{"type":"callback","title":"Stdio.Syscall.set_pdeathsig/0","doc":"Operations required to terminate a process if the parent exits.","ref":"Stdio.Syscall.html#c:set_pdeathsig/0"},{"type":"function","title":"Stdio.Syscall.setproctitle/2","doc":"The default implementation for `c:setproctitle/2`\n\nUses `:prx.setproctitle/2` on all platforms.","ref":"Stdio.Syscall.html#setproctitle/2"},{"type":"callback","title":"Stdio.Syscall.setproctitle/2","doc":"Set the process title.","ref":"Stdio.Syscall.html#c:setproctitle/2"},{"type":"function","title":"Stdio.Syscall.subprocess/1","doc":"The default implementation for `c:subprocess/1`\n\n`fork(2)` a subprocess from the supervisor.","ref":"Stdio.Syscall.html#subprocess/1"},{"type":"callback","title":"Stdio.Syscall.subprocess/1","doc":"Fork a subprocess.","ref":"Stdio.Syscall.html#c:subprocess/1"},{"type":"function","title":"Stdio.Syscall.subreaper/1","doc":"The default implementation for `c:subreaper/1`\n\nNo changes are made to the process: background processes will not\nbe terminated.","ref":"Stdio.Syscall.html#subreaper/1"},{"type":"callback","title":"Stdio.Syscall.subreaper/1","doc":"Set process as init(1) for descendent processes.","ref":"Stdio.Syscall.html#c:subreaper/1"},{"type":"module","title":"Stdio.Syscall.FreeBSD","doc":"System call portability for FreeBSD","ref":"Stdio.Syscall.FreeBSD.html"},{"type":"function","title":"Stdio.Syscall.FreeBSD.procfs/0","doc":"FreeBSD procfs support using the Linux compatible procfs.\n\nSee [linprocfs](https://www.freebsd.org/cgi/man.cgi?linprocfs(5))","ref":"Stdio.Syscall.FreeBSD.html#procfs/0"},{"type":"module","title":"Stdio.Syscall.Linux","doc":"System call portability for Linux.","ref":"Stdio.Syscall.Linux.html"},{"type":"function","title":"Stdio.Syscall.Linux.reap/2","doc":"Terminate descendents of a supervisor process.","ref":"Stdio.Syscall.Linux.html#reap/2"},{"type":"function","title":"Stdio.Syscall.Linux.reap/3","doc":"Terminate descendents of a process.\n\nreap signals subprocesses of a process identified by PID. If the\nprocess called `PR_SET_CHILD_SUBREAPER`, background and daemonized\nsubprocesses will also be terminated.\n\nNote: terminating subprocesses using this method is subject to race\nconditions:\n\n* new processes may have been forked\n\n* processes may have exited and unrelated processes assigned the PID\n\n* processes may ignore some signals\n\n* processes may not immediately exit after signalling","ref":"Stdio.Syscall.Linux.html#reap/3"},{"type":"module","title":"Stdio.Syscall.OpenBSD","doc":"System call portability for OpenBSD.","ref":"Stdio.Syscall.OpenBSD.html"},{"type":"exception","title":"Stdio.OpError","doc":"","ref":"Stdio.OpError.html"},{"type":"exception","title":"Stdio.StreamError","doc":"","ref":"Stdio.StreamError.html"},{"type":"extras","title":"Overview","doc":"# Stdio\n\n[![Package Version](https://img.shields.io/hexpm/v/stdio)](https://hex.pm/packages/stdio)\n[![Hex Docs](https://img.shields.io/badge/hex-docs)](https://hexdocs.pm/stdio/)\n\nStream standard I/O from system processes.\n\nReliably reap, restrict and isolate system tasks:\n[Stdio](https://github.com/msantos/stdio) is a control plane for processes.\n\n```mermaid\ngraph LR\n    B([beam]) -->|root| S0[Supervise]\n    B([beam]) -->|user| S1[Supervise]\n    S0 --> |user|I0[[init]]\n    S0 --> |user|I1[[init]]\n    subgraph namespace0\n    I0 --> C0[[echo 'Hello']]\n    end\n    subgraph namespace1\n    I1 --> C1[[echo 'System working?']]\n    end\n    S1 --> C2[[echo 'Seems to be.']]\n```","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"`Stdio` is an Elixir library. The package can be installed by adding\n`stdio` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:stdio, \"~> 0.4.4\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Overview","doc":"```elixir\niex> Stdio.stream!(\"echo test\") |> Enum.to_list()\n[stdout: \"test\\n\", exit_status: 0]\n```\n\nCommands use `/bin/sh -c`:\n\n```elixir\niex> Stdio.stream!(\"pstree $$\") |> Enum.to_list()\n[stdout: \"sh---pstree\\n\", exit_status: 0]\n```\n\n```mermaid\ngraph LR\n    B([beam]) -->S[Supervise]\n    S --> I[[sh]]\n    I --> C[[pstree]]\n```","ref":"readme.html#usage"},{"type":"extras","title":"Pipes - Overview","doc":"Streams can be piped into processes:\n\n```elixir\niex> [\"let\", \"it\", \"crash\"] |> Stdio.pipe!(\"tr '[a-z]' '[A-Z]'\") |> Enum.to_list()\n[stdout: \"LETITCRASH\", exit_status: 0]\n```\n\n```mermaid\ngraph LR\n    B([beam]) -->|\"[let, it, crash]\"|S[Supervise]\n    S --> I[[sh]]\n    I --> |\"[let, it, crash]\"|C[[\"tr '[a-z]' '[A-Z]'\"]]\n    C --> O([\"LETITCRASH\"])\n```","ref":"readme.html#pipes"},{"type":"extras","title":"Argument List - Overview","doc":"Use an argv to execute a command without an intermediary shell\nprocess. `$PATH` is not consulted: the path to the executable is required.\n\n```elixir\niex> Stdio.stream!([\"/bin/echo\", \"test\"]) |> Enum.to_list()\n[stdout: \"test\\n\", exit_status: 0]\n```\n\n```mermaid\ngraph LR\n    B([beam]) -->S[Supervise]\n    S --> C[[/bin/echo test]]\n```","ref":"readme.html#argument-list"},{"type":"extras","title":"Background Processes - Overview","doc":"Background and daemonized processes are reaped when the foreground\nprocess exits:\n\n```elixir\niex> Stdio.stream!(\"sleep 131 & sleep 111 & echo $$\") |> Enum.to_list()\n[stdout: \"25723\\n\", exit_status: 0]\niex> Stdio.Procfs.children(25723)\n[]\n```\n\n```mermaid\ngraph LR\n    B([beam]) -->S[Supervise]\n    S --- I[[/bin/sh]]\n    I -.- C0[[sleep 131]]\n    I -.- C1[[sleep 111]]\n    I --- C2[[echo $$]]\n    S --> C0\n    S --> C1\n```","ref":"readme.html#background-processes"},{"type":"extras","title":"Setuid Binaries - Overview","doc":"Setuid processes are disabled by default to prevent unkillable processes.\n\n```elixir\niex> Stdio.stream!(\"ping -c 1 8.8.8.8\") |> Enum.to_list()\n[stderr: \"ping: ssend socket: Operation not permitted\\n\", exit_status: 71]\n```\n\nThe process can escalate privileges by using the `setuid` option:\n\n```elixir\niex> Stdio.stream!(\"ping -c 1 8.8.8.8\", Stdio.Process, setuid: true) |> Enum.to_list()\n[\n  stdout: \"PING 8.8.8.8 (8.8.8.8): 56 data bytes\\n64 bytes from 8.8.8.8: icmp_seq=0 ttl=116 time=1.726 ms\\n\",\n  stdout: \"\\n--- 8.8.8.8 ping statistics ---\\n1 packets transmitted, 1 packets received, 0.0% packet loss\\nround-trip min/avg/max/stddev = 1.726/1.726/1.726/0.000 ms\\n\",\n  exit_status: 0\n]\n```","ref":"readme.html#setuid-binaries"},{"type":"extras","title":"Privileges - Overview","doc":"> [!IMPORTANT]\n>\n> Some behaviours may require running system processes as the root user.\n>\n> For setup, see\n> [Stdio.setuid/0](https://hexdocs.pm/stdio/Stdio.html#setuid/0).\n\nBehaviours may change the root filesystem for the process. The default\n`chroot(2)` directory hierarchy can be created by running:\n\n```elixir\niex> Stdio.Container.make_chroot_tree!()\n```","ref":"readme.html#privileges"},{"type":"extras","title":"Process Isolation - Overview","doc":"Behaviours can implement process restrictions or process isolation. For\nexample, by default the `Stdio.Rootless` behaviour does not have network\naccess:\n\n```elixir\niex> Stdio.stream!(\"ip -br addr show\", Stdio.Rootless) |> Enum.to_list()\n[stdout: \"lo               DOWN           \\n\", exit_status: 0]\n```\n\n```mermaid\ngraph LR\n    B([beam]) -->|user| S[Supervise]\n    S --> |user|I[[init]]\n    subgraph \"user/network namespace\"\n    I --> C[[ip -br addr show]]\n    end\n```","ref":"readme.html#process-isolation"},{"type":"extras","title":"Linux Container - Overview","doc":"The `Stdio.Container` behaviour also disables network access:\n\n```elixir\niex> Stdio.stream!(\"ping -c 1 8.8.8.8\", Stdio.Container, setuid: true) |> Enum.to_list()\n[stderr: \"ping: connect: Network is unreachable\\n\", exit_status: 2]\n```\n\n```mermaid\ngraph LR\n    B([beam]) -->|root| S[Supervise]\n    S --> |user|I[[sh]]\n    subgraph \"network namespace\"\n    I --> C[[ping -c 1 8.8.8.8]]\n    end\n```\n\nIf `setuid` is allowed and the `host` network is shared, `ping` works\nas expected:\n\n```elixir\niex> Stdio.stream!(\"ping -c 1 8.8.8.8\", Stdio.Container, setuid: true, net: :host) |> Enum.to_list()\n[\n  stdout: \"PING 8.8.8.8 (8.8.8.8) 56(84) bytes of data.\\n64 bytes from 8.8.8.8: icmp_seq=1 ttl=115 time=32.4 ms\\n\\n--- 8.8.8.8 ping statistics ---\\n1 packets transmitted, 1 received, 0% packet loss, time 0ms\\nrtt min/avg/max/mdev = 32.390/32.390/32.390/0.000 ms\\n\",\n  exit_status: 0\n]\n```\n\n```mermaid\ngraph LR\n    B([beam]) -->|root| S[Supervise]\n    S --> |user|I[[sh]]\n    subgraph \"namespace (net shared with parent)\"\n    I --> C[[ping -c 1 8.8.8.8]]\n    end\n```","ref":"readme.html#linux-container"},{"type":"extras","title":"FreeBSD Jails - Overview","doc":"FreeBSD Jails work in a similar way. An argv is used because the process\nroot directory has been changed to `/rescue`, a directory containing\nstatically linked binaries. By default `setuid` and `net` are disabled:\n\n```elixir\niex> Stdio.stream!([\"./ping\", \"-c\", \"1\", \"8.8.8.8\"], Stdio.Jail, path: \"/rescue\") |> Enum.to_list()\n[stderr: \"ping: ssend socket: Protocol not supported\\n\", exit_status: 71]\n```\n\n```mermaid\ngraph LR\n    B([beam]) -->|root| S[Supervise]\n    S --> |user|I[[sh]]\n    subgraph \"jail\"\n    I --> C[[ping -c 1 8.8.8.8]]\n    end\n```","ref":"readme.html#freebsd-jails"},{"type":"extras","title":"Examples - Overview","doc":"","ref":"readme.html#examples"},{"type":"extras","title":"inetd - Overview","doc":"The following code is an example of an `inetd(8)` service using forked system\nprocesses to handle client requests. It is adapted from the [Task and\ngen_tcp](https://elixir-lang.org/getting-started/mix-otp/task-and-gen-tcp.html)\necho server in the Elixir documentation.\n\nTo run from iex:\n\n```elixir\niex> Inetd.Server.start([\n...> %{port: 7070, command: \"cat -n\"},\n...> %{ip: \"127.0.0.1\", port: 7071, command: \"stdbuf -o0 tr [A-Z] [a-z]\"}\n...> ])\n```\n\n```elixir\ndefmodule Inetd.Server do\n  require Logger\n\n  def start(spec) do\n    children =\n      spec\n      |> Enum.map(fn %{port: port, command: command} = m ->\n        {:ok, ip} = :inet_parse.address(String.to_charlist(Map.get(m, :ip, \"::\")))\n        behaviour = Map.get(m, :behaviour, Stdio.Rootless)\n\n        Supervisor.child_spec(\n          {Task, fn -> Inetd.Server.accept(ip, port, command, behaviour) end},\n          id: {ip, port},\n          restart: :permanent\n        )\n      end)\n\n    children = [{Task.Supervisor, name: Inetd.TaskSupervisor}] ++ children\n\n    opts = [strategy: :one_for_one, name: Inetd.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\n\n  @doc \"\"\"\n  Starts accepting connections on the given `port`.\n  \"\"\"\n  def accept(ip, port, command, behaviour) do\n    {:ok, socket} =\n      :gen_tcp.listen(\n        port,\n        [:binary, packet: :line, active: false, reuseaddr: true, ip: ip]\n      )\n\n    Logger.info(\"Accepting connections on port #{port}\")\n    loop_acceptor(socket, command, behaviour)\n  end\n\n  defp loop_acceptor(socket, command, behaviour) do\n    {:ok, client} = :gen_tcp.accept(socket)\n\n    {:ok, pid} =\n      Task.Supervisor.start_child(Inetd.TaskSupervisor, fn ->\n        serve(client, command, behaviour)\n      end)\n\n    :ok = :gen_tcp.controlling_process(client, pid)\n    loop_acceptor(socket, command, behaviour)\n  end\n\n  defp serve(socket, command, behaviour) do\n    Stream.resource(\n      fn -> socket end,\n      fn socket -> read_line(socket) end,\n      fn socket -> :gen_tcp.close(socket) end\n    )\n    |> Stdio.pipe!(command, behaviour)\n    |> Stream.transform(socket, fn data, socket ->\n      write_line(data, socket)\n    end)\n    |> Stream.run()\n  end\n\n  defp read_line(socket) do\n    case :gen_tcp.recv(socket, 0) do\n      {:ok, data} ->\n        {[data], socket}\n\n      {:error, _} ->\n        {:halt, socket}\n    end\n  end\n\n  defp write_line({:stdout, line}, socket) do\n    case :gen_tcp.send(socket, line) do\n      :ok ->\n        {[], socket}\n\n      {:error, _} ->\n        {:halt, socket}\n    end\n  end\n\n  defp write_line({:stderr, line}, socket) do\n    case :gen_tcp.send(socket, line) do\n      :ok ->\n        {[], socket}\n\n      {:error, _} ->\n        {:halt, socket}\n    end\n  end\n\n  defp write_line({msg, _} = data, socket) when msg in [:exit_status, :termsig] do\n    Logger.info(\"#{inspect(data)}\")\n    {:halt, socket}\n  end\nend\n```","ref":"readme.html#inetd"},{"type":"extras","title":"Documentation - Overview","doc":"Documentation is available on [hexdocs](https://hexdocs.pm/stdio/).","ref":"readme.html#documentation"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}